#ifndef _ASTRING_H
#define _ASTRING_H

#include <iostream.h>
#include "bool.h"

//*******************************************************************
//  A string class consistent with a subset of the standard C++
//  string class as defined in the draft ANSI standard.
//*******************************************************************

extern const int npos;  // Returned by find(...) when the character
                        //   or substring is not found

class astring {

  public:

  // Constructors/destructor

    astring();                        // Constructs an empty string
    astring (const char *s);          // Constructs from a literal string
    astring (const astring &str);     // Copy constructor
    ~astring();                       // Destructor

  // Assignment

    const astring &operator= (const astring &str); // Assign str
    const astring &operator= (const char *s);      // Assign s
    const astring &operator= (char ch );           // Assign ch

  // Accessors

    int length() const;
       // Returns the current length (number of characters)

    int find (char ch) const;
       // Returns position of the first occurrence of the character ch

    int find (const astring &str) const;
       // Returns position of the first occurrence of the substring str

    astring substr (int pos, int len) const;
       // Returns the substring of len chars starting at pos

    const char *c_str( ) const;
       // Returns the pointer to the corresponding null-terminated string

  // Subscipts (with range checking)

    char  operator[] (int k) const;
    char &operator[] (int k);

  // Concatenation

    const astring &operator+= (const astring &str); // Appends a str
    const astring &operator+= (char ch);            // Appends a char

  private:
      int myLength;        // length of string (# of characters)
      int myCapacity;      // capacity of string
      char *myCstring;     // buffer for characters         
};

// The following free (non-member) functions operate on strings:

//      I/O functions

ostream &operator<< (ostream &os, const astring &str);
istream &operator>> (istream &is, astring &str); 
istream &getline (istream & is, astring &str);  
          
// Relational operators:

bool operator== (const astring &lhs, const astring &rhs);  
bool operator!= (const astring &lhs, const astring &rhs);  
bool operator<  (const astring &lhs, const astring &rhs);  
bool operator<= (const astring &lhs, const astring &rhs);  
bool operator>  (const astring &lhs, const astring &rhs);  
bool operator>= (const astring &lhs, const astring &rhs);  

// Concatenation operator +

astring operator+ (const astring &lhs, const astring &rhs);
astring operator+ (char ch, const astring & str);
astring operator+ (const astring &str, char ch);

#endif

