/* DICT.CPP

    This program works as an on-line dictionary. It reads a word
    and shows its translation.  The program loads the dictionary
    data from the DICT.DAT file.
 
    Author: M. Webster
    Rev. 1.0     09/20/98
*/

#include <fstream.h>
#include <string.h>
#include "bool.h"

struct ENTRY {
    char word[20];
    char translation[20];
};

const int MAXWORDS = 1000;  // Max number of words in the dictionary

// Function prototypes
ENTRY *LoadDictionary(char filename[]);
bool FoundWord(ENTRY dict[], char word[], char translation[]);

/****************************************************************/
/**************            Main Program             *************/
/****************************************************************/

void main()

{
    ENTRY *dict;
    char word[20], translation[20];

    // Load the dictionary from the file

    dict = LoadDictionary("DICT.DAT");
    if (!dict) {
        cout << "*** Cannot load dictionary ***\n";
        return;
    }

    // Translate words

    bool quit = false;
    while (!quit) {
        cout << "Enter a word or 'q' to quit ==> ";
        cin >> word;             // Read one word and
        cin.ignore(80, '\n');    //   skip the rest of the line
        if (stricmp(word, "q") == 0)
            quit = true;
        else if (FoundWord(dict, word, translation))
            cout << translation << "\n\n";
        else
            cout << word << " -- not in the dictionary.\n\n";
    }

    // Clean up

    delete [] dict;
}

/****************************************************************/
/**************              Functions              *************/
/****************************************************************/

ENTRY *LoadDictionary(char filename[])

// Allocates a dictionary array and reads entries from a file.
// Returns a pointer to the dictionary.

{
    ENTRY *dict;

    // Open dictionary file
    ifstream inp(filename);
    if (!inp)
        return 0;

    // Allocate memory for the dictionary
    dict = new ENTRY[MAXWORDS];
    if (!dict)
        return 0;

    // Read and display header line
    char line[80];
    inp.getline(line, 80);
    cout << line << endl;

    // Read words and translations into the dictionary array
    int i = 0;
    while (i < MAXWORDS &&
           inp >> dict[i].word >> dict[i].translation) {
        inp.ignore(80, '\n'); // Skip the rest of the line
        i++;
    }

    // Add a "sentinel" (empty word) at the end of the dictionary.
    *dict[i].word = '\0';

    // Report number of entries
    cout << " (" << i << " words)\n\n";

    return dict;
}

/****************************************************************/

bool FoundWord(ENTRY *dict, char word[], char translation[])

// Finds a word in the dictionary.
//  dict -- pointer to the dictionary
//  word -- word to translate
//  translation -- returned translation from the dictionary.
// Returns true if the word has been found, false otherwise.

{
    bool found = false;
    for (int i = 0;   !found && *dict[i].word != '\0';   i++) {
        if (stricmp(dict[i].word, word) == 0) {
            strcpy(translation, dict[i].translation);
            found = true;
        }
    }
    return found;
}

 
