// MENU.CPP
//
// This program illustrates the use of simple one-character
//   commands.  The program displays a menu, accepts a command
//   and executes it.  The application is an embryonic
//   "Inventory Control System" which, at this stage of development,
//   only maintains a list of "inventory items" (integers).
//   Items are stored in an integer array. The "quantity" is
//   not supported -- it is set to 1 for all inventory items.
//
// Author: Bill Wares
//

#include <iostream.h>
#include <iomanip.h>
#include <ctype.h>                 // Declares toupper(ch)

const int MAXITEMS = 10;           // Maximum number of items.
int nItems = 0;                    // Current number of items.
int inventory[MAXITEMS];           // Array of items.

const int QUANTITY = 1;            // Quantity for each item is set
                                   //   to 1 in this prelim. version.

// Function prototypes:

void ShowInventory (int partNum);
void AddInventory (int partNum);
void DeleteInventory (int partNum);
void ListInventory ();

//****************************************************************
//****************             main            *******************
//****************************************************************

void main()

{
    char cmd;
    int partNum;

    cout << "\n          One Of Each, Inc.\n";
    cout << "      Inventory Control System\n";

    for(;;) {  // Repeat (until break)

        // Show the menu and prompt:

        cout << "\n";  // Output a blank line

        cout << "\t (S)how inventory item\n";      // '\t' is tab
        cout << "\t (A)dd item\n";
        cout << "\t (D)elete item\n";
        cout << "\t (L)ist inventory\n";
        cout << "\t (Q)uit\n";
        cout << endl;
        cout << "Next command ==> ";

        // Accept command:

        cin >> cmd;             // Read one char.
        cin.ignore(80, '\n');   // Skip remaining input (up to 80
                                //   chars)to the end of the line.
        cmd = toupper(cmd);     // Convert letters to upper case
                                //   to allow lower case input
                                //   for commands (for convenience).

        // Quit if 'Q'

        if (cmd == 'Q')
            break;                  // Quit processing commands

        cout << "\n\n****************************************\n";

        // Process command:

        switch (cmd) {

          case 'S':           // Show inventory item information

            cout << "Part number: ";
            cin >> partNum;
            ShowInventory(partNum);
            break;

          case 'A':           // Add inventory item

            cout << "Part number: ";
            cin >> partNum;
            AddInventory(partNum);
            break;

          case 'D':           // Delete inventory item

            cout << "Part number: ";
            cin >> partNum;
            DeleteInventory(partNum);
            break;

          case 'L':           // Display the inventory list

            ListInventory();
            break;

          default:

            cout << "Invalid command.\n";
            break;
        }

        cout << "****************************************\n";
    }
}

//****************************************************************
//****************          Functions          *******************
//****************************************************************

int FindInventory (int partNum)

// Finds the part number, partNum, in the inventory array.
// Returns its index, if found, -1 otherwise.

{
    ...
}

//****************************************************************

void ShowInventory(int partNum)

// Displays inventory information for the given part number.

{
    int i;

    i = FindInventory(partNum);
    if (i >= 0)
        cout << "is available in quantity " << QUANTITY << ".\n";
    else
        cout << "not found.\n";
}

//****************************************************************

void AddInventory(int partNum)

// Adds the new inventory item with the specified part number,
//   partNum, to the inventory list.
//   Checks whether partNum is already in the list
//   and whether the list is full (nItems == MAXITEMS).
//   Displays an appropriate error message, or "ok" for success.

{
    ...
}

//****************************************************************

void DeleteInventory(int partNum)

// Deletes the item partNum from the inventory list, if it
//   is there.  Displays an appropriate message if partNum is not
//   in the list.

{
    ...
}

//****************************************************************

void ListInventory()

// Displays the inventory list.

{
    int i;

    cout << "\t PART NO.    QTY\n";
    cout << "\t--------------------\n";
    for (i = 0;   i < nItems;   i++) {
        ...
    }
}
