// NOTABS.CPP
// 
// This program prompts the user for the input file name,
//   reads the text file (up to 20000 chars), converts tabs
//   into spaces, and writes the modified text into
//   the output file.
//
// Author: Tabitha Spacek
// Rev 1.0

#include <fstream.h>

const int tabstop = 4;
const unsigned BUFSIZE = 20000;

int TabsToSpaces (char buf[], int n);

//****************************************************************

void main()

{
    char filename[40];
    int i, n;
    char *buf;

    // Open input file:
    cout << "Input file name: ";
    cin >> filename;

    ifstream inp(filename);
    if (!inp) {
        cout << "Cannot open " << filename << endl;
        return;
    }

    // Create output file:
    cout << "Output file name: ";
    cin >> filename;
    ofstream outp(filename);
    if (!outp) {
        cout << "Cannot create " << filename << endl;
        return;
    }

    // Allocate a character buffer:
    buf = new char[BUFSIZE];
    if (!buf) {
        cout << "Memory allocation error.\n";
        return;
    }

    // Read the input file into buf:
    n = 0;
    while (n < BUFSIZE && inp.get(buf[n]))
        n++;

    // Replace tabs:
    n = TabsToSpaces(buf, n);  // Returns the number of chars in the
                               //   modified text.

    // Write buf into the output file:
    for (i = 0;   i < n;   i++)
        outp.put(buf[i]);        

    // Release memory:
    delete [] buf;
}

//****************************************************************

int TabsToSpaces (char buf[], int n)

// Replaces each tab character in "buf" with the appropriate
//   number of spaces (to the next tab stop, as defined in the
//   "tabstop" constant).  n is the number of chars in buf.
// Returns the number of characters in the modified text.

{
    int n1;           // Number of chars in the modified text.
    char *buf1;       // Pointer to the modified text buffer.
    int i, col = 0;
    char ch;

    // Allocate a temporary array buf1[]:
    buf1 = new char[BUFSIZE];
    if (!buf1) {
        cout << "Memory allocation error in TabsToSpaces(...).\n";
        return 0;
    }

    // Copy characters from buf to buf1, replacing tabs
    //   (and increment n1 accordingly):
    for (i = 0, n1 = 0;   i < n && n1 < BUFSIZE;   i++) {
        ch = buf[i];
        if (ch == '\n') {                // If newline ...
            buf1[n1++] = ch;             // ... copy it into buf1
            col = 0;                     //   and reset col to 0.
        }
        else if (ch == '\t') {           // If tab ...
            do {                         // ... pad with spaces
                buf1[n1++] = ' ';        //   until the next
                col++;                   //   tab stop.
            } while (n1 < BUFSIZE && col % tabstop != 0);
        }
        else {                           // If any other char ...
            buf1[n1++] = ch;             // ... copy it into buf1
            col++;                       //   and increment col.
        }
    }

    // Copy buf1 back to buf[]:
    for (i = 0;   i < n1;   i++)
        buf[i] = buf1[i];

    // Free buf1:
    delete [] buf1;

    return n1; 
}

