// VECTOR.H

#include <iostream.h>
#include <iomanip.h>

const int SIZE = 10;

class VECTOR {

  private:

    int *buf;

  public:

        // CANONICAL FEATURES

    // *** Default constructor ***
    VECTOR();

    // *** Copy constructor ***
    VECTOR (const VECTOR &a_vector);

    // *** An assignment operator ***
    const VECTOR &operator= (const VECTOR &a_vector);

    // *** Destructor ***
    ~VECTOR();

        // OPTIONAL FEATURES

    // *** Reasonable automatic conversions ***
    const VECTOR &operator= (const int a[]);

    // *** Reasonable arithmetic operators ***
    const VECTOR &operator+= (const VECTOR &a_vector);
    
    // *** << and >> for stream I/O ***

  friend ostream &operator<< (ostream &outp, const VECTOR &vector);
  friend istream &operator>> (istream &inp,  VECTOR &vector);

};

////////////////////////////////////////////////////////////////
// VECTOR.CPP

//#include "vector.h"

//        CANONICAL FEATURES

// *** Default constructor ***

VECTOR::VECTOR()

{
    buf = new int[SIZE];
    for (int i = 0;   i < SIZE;   i++)
        buf[i] = 0;
}

// *** Copy constructor ***

VECTOR::VECTOR (const VECTOR &a_vector)

{
    if (&a_vector != this) {
        buf = new int[SIZE];
        for (int i = 0;   i < SIZE;   i++)
            buf[i] = a_vector.buf[i];
    }
}

// *** Assignment operator ***

const VECTOR &VECTOR::operator= (const VECTOR& a_vector)

{
    if (&a_vector != this) {
        for (int i = 0;   i < SIZE;   i++)
            buf[i] = a_vector.buf[i];
    }
    return *this;
}

// *** Destructor ***

VECTOR::~VECTOR()

{
    delete [] buf;
}

//         OPTIONAL FEATURES

// *** Automatic conversion from int[] to VECTOR ***

const VECTOR &VECTOR::operator= (const int a[])

{
    for (int i = 0;   i < SIZE;   i++)
        buf[i] = a[i];
    return *this;
}

// *** Arithmetic operators for numeric classes ***

const VECTOR &VECTOR::operator+= (const VECTOR& a_vector)

{
    for (int i = 0;   i < SIZE;   i++)
        buf[i] += a_vector.buf[i];
    return *this;
}

// *** << and >> for stream I/O ***

ostream &operator<< (ostream& outp, const VECTOR &vector)

{
    outp << setw(6); 
    for (int i = 0;   i < SIZE;   i++)
        outp << vector.buf[i] << ' ';
    return outp;
}

istream &operator>> (istream& inp,  VECTOR &vector)

{
    for (int i = 0;   i < SIZE;   i++)
        inp >> vector.buf[i];
    return inp;
}

////////////////////////////////////////////////////////////////
// TEST.CPP

//#include "vector.h"

void main()

{
    VECTOR v1, v2;

    cout << "Show values from the default constructor:\n"
         << v1 << endl;

    cout << "Enter a vector (10 integers) ==> ";
    cin >> v1;
    cout << "Show values entered from cin into v1:\n"
         << v1 << endl;

    v2 = v1;
    cout << "The same values in v2 after assignment v2 = v1:\n"
         << v2 << endl;

    VECTOR v3 = v2;
    cout << "The same values in v3 from the copy constructor:\n"
         << v3 << endl;

    int a[] = {9, 8, 7, 6, 5, 4, 3, 2, 1, 0};
    v3 = a;
    cout << "Show values after automatic conversion:\n"
         << v3 << endl;

    v3 += v2;
    cout << "Test += operator:\n"
         << v3 << endl;
}

