// LIST.H

const int SIZE = 100;

struct NODE {
    char data[SIZE];
    NODE *next;
};

// *** Linked List class ***

class LIST {

  private:

    NODE *head;
    NODE *tail;

  public:

    LIST () {head = tail = 0;}
    void Insert (const char str[]);
//    ...
    ~LIST();

  friend class LISTITER;

};

// *** Iterator for LIST ***

class LISTITER {

  private:

    NODE *current;

  public:

    // Constructor with initialization to a LIST object:
    LISTITER(LIST &list) {current = list.head;}

    // Copy constructor:
    LISTITER(LISTITER &listiter) {current = listiter.current;}

    // Returns the pointer to current data and 
    //  advances the current pointer:
    char *Next();
};

////////////////////////////////////////////////////////////////
// LIST.CPP

#include <string.h>
//#include "list.h"

// *** LIST member functions ***

void LIST::Insert (const char str[])

{
    NODE *newnode = new NODE;
    strcpy(newnode->data, str);
    newnode->next = 0;
    if (tail)
        tail->next = newnode;
    else
        head = newnode;
    tail = newnode;
}

LIST::~LIST() // Deletes all nodes

{
    NODE *nextnode;

    while (head) {
        nextnode = head->next;
        delete head;
        head = nextnode;
    }
}

//...

// *** LISTITER member functions ***

char *LISTITER::Next()

// Returns the current data and advances the current pointer

{
    char *str = 0;
    if (current) {
        str = current->data;
        current = current->next;
    }
    return str;
}

////////////////////////////////////////////////////////////////
// TEST.CPP

#include <iostream.h>
#include <string.h>
//#include "list.h"

void Print(LIST &list)

// Prints all elements in the list by using one iterator.

{
    LISTITER listiter(list);
    char *str;

    while ((str = listiter.Next()) != 0)
        cout << str << ' ';
    cout << endl;    
}

//****************************************************************

void PrintDuplicates(LIST &list)

// Prints all elements in the list that have duplicates
//   by using two nested iterators.

{
    cout << "Duplicates:\n";

    LISTITER listiter1(list);
    char *str1, *str2;

    while ((str1 = listiter1.Next()) != 0) {
        LISTITER listiter2 = listiter1;
        while ((str2 = listiter2.Next()) != 0) {
            if (strcmp(str1, str2) == 0) {
                cout << '[' << str1 << ']' << endl;
                break;
            }
        }
    }
}

//****************************************************************

void main()

{
    LIST list;

    list.Insert("a");
    list.Insert("rose");
    list.Insert("is");
    list.Insert("a");
    list.Insert("rose");
    Print(list);
    cout << endl;
    PrintDuplicates(list);
}

