// MORSE.CPP

#include <ctype.h>
#include "morse.h"

MORSECODE TELEGRAPH::table[40] = {
  {'A', ".-"},     {'B', "-..."},    {'C', "-.-."},   {'D', "-.."},
  {'E', "."},      {'F', "..-."},    {'G', "--."},    {'H', "...."},
  {'I', ".."},     {'J', ".---"},    {'K', "-.-"},    {'L', ".-.."},
  {'M', "--"},     {'N', "-."},      {'O', "---"},    {'P', ".--."},
  {'Q', "--.-"},   {'R', ".-."},     {'S', "..."},    {'T', "-"},
  {'U', "..-"},    {'V', "...-"},    {'W', ".--"},    {'X', "-..-"},
  {'Y', "-.--"},   {'Z', "--.."},
  {'0', "-----"},  {'1', ".----"},   {'2', "..---"},  {'3', "...--"},
  {'4', "....-"},  {'5', "....."},   {'6', "-...."},  {'7', "--..."},
  {'8', "---.."},  {'9', "----."},
  {'.', ".-.-.-"}, {',', "--..--"},  {'?', "..--.."},
  {'\0', "END"}
};

static const char DOT = '.';
static const char DASH = '-';

TNODE *TELEGRAPH::root = 0;        // The root of the decoding tree.

void TELEGRAPH::Open()

// Builds the decoding tree for TELEGRAPH from the table above.
//   Each Morse code character will correspond to a path in the tree
//   from the root to a node: at a "dot" go left, at a  "dash" go
//   right. The node at the end of the path holds the symbol
//   for that character.

{
    TNODE *node, *nextnode;
    int i;
    char *dd;     // dd points to a string of dots and dashes.

    // Create the root node:
    root = new TNODE;
    if (!root) return;
    root->symbol = ' ';

    // For each table entry create a path in the decoding tree.
    //   table[i].symbol == '\0' is used as a sentinel to mark
    //   the end of the table.
    for (i = 0;   table[i].symbol;   i++) {
        // Build path:
        node = root;
        for (dd = table[i].code;   *dd;   dd++) {
            // "dd" points to the next char in the Morse code
            ...
            ...
            ...
        }
        node->symbol = table[i].symbol;     // "node" points to the
                                            //    end of the path.
    }
}

void TELEGRAPH::DestroyTree(TNODE *node)

{
            ...
            ...
            ...
}

void TELEGRAPH::Close()

// Destroys the decoding tree by calling the recursive function
//   above.

{
    DestroyTree(root);
    root = 0;
}

void TELEGRAPH::Encode(char text[], char morse[])

// Converts text into Morse code.

{
    int i;
    char c, *t, *dd;  // t points to text;
                      // dd points to a string of dots and dashes.

    for (t = text;   *t;   t++) {
        c = toupper(*t);

        // If space, add a space to the morse string:
        if (c == ' ') {
            *morse++ = ' ';
            continue;
        }

        // Find this symbol in the MORSECODE table;
        //   skip this symbol if not found:
        for (i = 0;   table[i].symbol;   i++)
            if (table[i].symbol == c) break;
        if (!table[i].symbol) continue;

        // Copy its code into the morse string:
        dd = table[i].code;
        while (*dd) *morse++ = *dd++;

        // Add one space to separate letters:
        *morse++ = ' ';
    }        
    *morse = '\0';
}

void TELEGRAPH::Decode(char morse[], char text[])

// Converts Morse code into text.

{
    char *dd;
    TNODE *node;

    node = root;
    // For each char in the encoded message (can be
    //   a dot, a dash, or a space):
    for (dd = morse;   *dd;   dd++) {
        // Go down the tree as long as you get dots or dashes.
        //   When you encounter a space, take the symbol from
        //   the node, add it to the text string, and return
        // to the root.
        ...
        ...
        ...
    }
    *text = '\0';
}

