// RBQUEUE.CPP
//
// Queue implemented as a ring buffer.

#include "rbqueue.h"

STATUS RBQUEUE::Insert (char c)

// Appends c at the end of the buffer.
// Returns OK if successful, FAILED if the queue is full.

{
    int next = Next(rear);
    if (next == front) return FAILED;
    buf[rear] = c;
    rear = next;
    return OK;
}

STATUS RBQUEUE::Remove (char &c)

// Retrieves and removes the element from the front of the buffer.
// Returns OK if successful, FAILED if the queue is empty.

{
    if (front == rear) return FAILED;
    c = buf[front];
    front = Next(front);
    return OK;
}

//****************************************************************

STATUS RBQUEUE::Insert (char str[])

// Appends a null-terminated string to the buffer.
// Returns OK if successful, FAILED if the string overflows
//   the queue.

{
    STATUS status;
    do {
        status = Insert(*str);
    } while (*str++ && status == OK);

    return status;
}

STATUS RBQUEUE::Remove (char str[])

// Retrieves a null-terminated string from the buffer.
// Returns OK if successful, FAILED if the string does not
//   end with a null.

{
    STATUS status;
    do {
        status = Remove(*str);
    } while (*str++ && status == OK);

    return status;
}

