// RBQUEUE.H
//
// Queue implemented as a ring buffer.

#ifndef _RBQUEUE_H_
#define _RBQUEUE_H_

#include "bool.h"    // defines bool type
#include "status.h"  // defines STATUS type

const int BUFSIZE = 256;


class RBQUEUE {        // Ring buffer for characters 

  private:

    char buf[BUFSIZE];
    int front;
    int rear;

    int Next(int index) {  // Calculates and returns the value
                           //   of the next index with wrap-around.
        if (++index == BUFSIZE) index = 0;
        return index;
    }

  public:

    RBQUEUE() {front = rear = 0;}
    ~RBQUEUE() {}
    STATUS Insert (char c);
    STATUS Remove (char &c);
    STATUS Insert (char str[]);   // Insert a null-terminated string.
    STATUS Remove (char str[]);   // Remove a null-terminated string.
    bool IsEmpty() {return (front == rear);}
    bool IsFull() {return (Next(rear) == front);}
};

#endif    // _RBQUEUE_H_    

