// SORTS.CPP
//
// Implementations of the binary search and sorting algorithms.
//

//****************************************************************
//***********              Binary search            **************
//****************************************************************

template <class SOMETYPE>
int BinarySearch(SOMETYPE a[], int n, SOMETYPE target)

// Performs a binary search in the array "a".
//   "n" is the number of elements in the array.
//   The elements in the array must be arranged in
//   ascending order.
//   Looks for a value equal to "target".
//   (the ==, < and > operators as well as assignment must be
//   defined for SOMETYPE.)
// Returns: the location of "target" in "a", if found; -1 otherwise.

{
    int location = -1;
    int left = 0, right = n-1, middle;

    while (left <= right) {
        middle = (left + right) / 2;
        if (target > a[middle])
            left = middle + 1;
        else if (target < a[middle])
            right = middle - 1;
        else {
            // if (target == a[middle])
            location = middle;
            break;
        }
    }
    return location;
}

//****************************************************************
//***********              Selection sort           **************
//****************************************************************

template <class SOMETYPE>
void Swap(SOMETYPE &x, SOMETYPE &y)

{
    SOMETYPE temp = x; x = y; y = temp;
}

template <class SOMETYPE>
void SelectionSort(SOMETYPE a[], int n)

{
    int i, iMax;

    while (n > 1) {

        // Find the largest element:
        for (iMax = 0, i = 1;   i < n;   i++)
            if (a[i] > a[iMax]) iMax = i;

        // Swap it with the last element:
        Swap(a[iMax], a[n-1]);

        n--;
    }
}

//****************************************************************
//***********              Insertion sort           **************
//****************************************************************

template <class SOMETYPE>
void InsertionSort(SOMETYPE a[], int n)

{
    int i, j;
    SOMETYPE aCurrent;

    for (i = 1;   i < n;   i++) {

        // Save the current element:
        aCurrent = a[i];

        // Find location j where it should be inserted
        //   among the first i-1 elements:
        for (j = 0;   j < i;  j++)
            if (a[j] >= aCurrent) break;

        // Shift all elements between j and i one place to the right:
        for (int k = i-1;  k >= j;  k--)
            a[k+1] = a[k];

        // Insert saved element where it belongs:
        a[j] = aCurrent;
    }
}

//****************************************************************

template <class SOMETYPE>
void InsertionSort2(SOMETYPE a[], int n)

// A variation of insertion sort
//   with scanning the sublist backwards from i-1 down to 0
 
{
    int i, j;
    SOMETYPE aCurrent;

    for (i = 1;   i < n;   i++) {

        // Save the current element:
        aCurrent = a[i];

        // Going backwards from a[i-1], shift elements to the
        //   right until you find an element a[j] <= aCurrent:
        for (j = i-1;   j >= 0;  j--) {
            if (a[j] <= aCurrent) break;
            a[j+1]  = a[j];
        }

        // Insert saved element after a[j]:
        a[j+1] = aCurrent;
    }
}

//****************************************************************
//***********              Bubble sort              **************
//****************************************************************

template <class SOMETYPE>
void BubbleSort(SOMETYPE a[], int n)

// Quits early when the array is confirmed to be in order.

{
    int i, disorder = n;

    while (disorder) {

        disorder = 0;

        // Bubble up and mark any disorder:
        //   (When n becomes 1, the "for" loop is not
        //    executed, "disorder" remains 0, and the "while"
        //    loop is broken.)

        for (i = 1;   i < n;   i++) {
            if (a[i] < a[i-1]) {
                Swap(a[i], a[i-1]);
                disorder++;
            }
        }
        n--;
    }
}

//****************************************************************
//***********               Mergesort               **************
//****************************************************************
// MERGESRT.H

template <class SOMETYPE>
class MERGESORT {

  private:

    SOMETYPE *work;
    inline void Swap(SOMETYPE &a, SOMETYPE &b);
    void Merge(SOMETYPE a1[], int n1, SOMETYPE a2[], int n2);

  public:

    // Constructor: allocates a work array
    MERGESORT(int n) { work = new SOMETYPE[n]; }

    // Destructor: frees the work array
    ~MERGESORT() { delete [] work; }

    // Recursive sort function:
    void Sort(SOMETYPE a[], int n);
};

//****************************************************************

template <class SOMETYPE>
void MERGESORT<SOMETYPE>::
       Sort(SOMETYPE a[], int n)

// Sorts array a of n elements in ascending order.

{
    int n1, n2;
    SOMETYPE *a2;

    if (n <= 2) {  // Base case:
        if (n == 2 && a[1] < a[0])
            Swap(a[0], a[1]);
    }
    else {         // Recursive case:
        n1 = n/2; n2 = n - n1;
        a2 = &a[n1];      // a2 points to the second half of the
                          //   array.
        Sort(a, n1);
        Sort(a2, n2);
        Merge(a, n1, a2, n2);
    }
}

//****************************************************************

template <class SOMETYPE>
void MERGESORT<SOMETYPE>::
       Merge(SOMETYPE a1[], int n1, SOMETYPE a2[], int n2)

// Merges sorted arrays a1 of length n1 and a2 of length n2
//   in sorted order in a1[].

{
    int n = n1 + n2;
    int i1, i2, j;

    // Merge two arrays into the work array:
    i1 = i2 = 0;
    for (j = 0;   j < n;   j++) {
        if (i1 < n1 && i2 < n2) {
            if (a1[i1] < a2[i2])
                work[j] = a1[i1++];
            else
                work[j] = a2[i2++];
        }
        else if (i1 < n1)
            work[j] = a1[i1++];
        else // if (i2 < n2)
            work[j] = a2[i2++];
    }

    // Copy work[] back to a1[]:
    for (j = 0;   j < n;   j++)
        a1[j] = work[j];
}

//****************************************************************

template <class SOMETYPE>
void MERGESORT<SOMETYPE>::
       Swap(SOMETYPE &a, SOMETYPE &b)
{
    SOMETYPE temp = a;  a = b;  b = temp;
}

//****************************************************************
//***********               Quicksort               **************
//****************************************************************
// QUICKSRT.H

template <class SOMETYPE>
class QUICKSORT {

  private:

    inline void Swap(SOMETYPE &a, SOMETYPE &b);

    // Returns the new location of the pivot element:
    int Split(SOMETYPE a[], int n);

  public:

    // Recursive sort function:
    void Sort(SOMETYPE a[], int n);
};

//****************************************************************

template <class SOMETYPE>
int QUICKSORT<SOMETYPE>::
      Split(SOMETYPE a[], int n)

// Takes one of the values (here a[n/2]) as "pivot"
//   and splits the array into two parts so that:
//      a[p] == pivot
//      a[i] <= pivot for i < p
//      a[j] >= pivot for j > p
// Returns p;

{
    int p = n/2;  // In this version the "pivot" value is the
                  //   middle element.

    SOMETYPE pivot = a[p];

    int i = 0, j = n-1;

    // Increment i if a[i] <= pivot; decrement j if a[j] >= pivot;
    //   otherwise swap a[i] and a[j] and advance both i and j:
    while (i <= j) {
        if (a[i] <= pivot)
            i++;
        else if (a[j] >= pivot)
            j--;
        else {
            Swap(a[i], a[j]);
            i++;
            j--;
        }
    }

    // Now a[i] and a[j] are adjacent elements; i > j.
    //   Swap a[i] or a[j] with the pivot, making sure that
    //   the swapped element remains on the same side of the pair:
    if (p < j) {
        Swap(a[p], a[j]);
        p = j;
    }
    else if (p > i) {
        Swap(a[p], a[i]);
        p = i;
    }
    return p;
}

//****************************************************************

#ifdef SIMPLIFIED

template <class SOMETYPE>
void QUICKSORT<SOMETYPE>::
       Sort(SOMETYPE a[], int n)

// Sorts array a of length n in ascending order.

{
    int p;

    if (n <= 2) {  // Base case:
        if (n == 2 && a[1] < a[0])
            Swap(a[0], a[1]);
    }
    else {         // Recursive case:
        p = Split(a, n);
        Sort(a, p);           // Sort the part from 0 to p-1.
        Sort(&a[p+1], n-p-1); // Sort the part from p+1 to n-1.
    }
}

#else

template <class SOMETYPE>
void QUICKSORT<SOMETYPE>::
       Sort(SOMETYPE a[], int n)

// Modified to assure the maximum recursion depth is O(log n).

{
    int p;

    while (n > 1) {
        if (n == 2) { // Base case
            if (a[1] < a[0]) Swap(a[0], a[1]);
            break;
        }
        else {  // Recursive case
            p = Split(a, n);
            if (p > n/2) {
                Sort(&a[p+1], n-p-1); // Process recursively the
                                      //   right part
                n = p;                // Continue iteratively with
                                      //   the left part
            }
            else {
                Sort(a, p);           // Process recursively the
                                      //   left part
                a += p+1;             // Continue iteratively with
                n -= p+1;             //   the right part
            }
        }
    }
}

#endif

//****************************************************************

template <class SOMETYPE>
void QUICKSORT<SOMETYPE>::
       Swap(SOMETYPE &a, SOMETYPE &b)
{
    SOMETYPE temp = a;  a = b;  b = temp;
}

//****************************************************************
//***********                Heapsort               **************
//****************************************************************
// HEAPSORT.H

template <class SOMETYPE>
class HEAPSORT {

  private:

    void Reheap(SOMETYPE a[], int i, int n);

  public:

    void Sort(SOMETYPE a[], int n);
};

//****************************************************************

template <class SOMETYPE>
void HEAPSORT<SOMETYPE>::
       Reheap(SOMETYPE a[], int i, int n)

//  Fixes the (sub)heap rooted at the node a[i], assuming that
//    all the nodes below it (all its descendants) already
//    satisfy the heap ordering property.
//  n is the largest possible index of an element in the heap:
//    a[j] is not in the heap when j > n.  

{
    SOMETYPE x = a[i];  // Save the root value.

    int iParent = i;
    i *= 2;             // Set i to left child.
    while (i <= n) {
        // Set i to the right child, if it is larger:
        if (i < n && a[i] < a[i+1]) i++;

        if (a[i] <= x) break;

        a[iParent] = a[i];     // Move the child up.
        iParent = i;           // a[iParent] is now vacant.
        i *= 2;                // i set to its left child.
    }
    a[iParent] = x;
}

//****************************************************************

template <class SOMETYPE>
void HEAPSORT<SOMETYPE>::
       Sort(SOMETYPE a[], int n)

// Sorts array a of length n in ascending order.

{
    // For heaps it is more convenient to index elements
    //  starting from 1, rather than 0:
    a--;

    // Fix heaps rooted in all nodes with children, going
    //   from the lowest-rightmost such node and proceeding
    //   left and up to the root.  a[i] is a leaf for any i > n/2,
    //   and a[n/2] is the parent of a[n], and therefore not a leaf.
    //   So we start with a[n/2].
    for (int i = n/2;   i >= 1;   i--)
        Reheap(a, i, n);

    // Remove elements from the top of the heap and store them
    //   in reverse order starting from the end of the array:
    SOMETYPE x;
    while (n > 1) {
        x = a[1]; a[1] = a[n]; a[n] = x;
        Reheap(a, 1, --n);
    }
}

//****************************************************************
//***********                Test program           **************
//****************************************************************

#include <iostream.h>

static void Copy (int buf1[], int buf2[])

{
    for (int i = 0;   i < 10;   i++) buf2[i] = buf1[i];
}

static void Print (int buf[])

{
    for (int i = 0;   i < 10;   i++) cout << buf[i] << ' ';
    cout << endl;
}

//****************************************************************

void main()

{
    int buf0[10], buf[10];
    int i;

    cout << "Enter 10 integers ==> ";
    for (i = 0;   i < 10;   i++) cin >> buf0[i];
    Print (buf0);

    Copy(buf0, buf);
    SelectionSort(buf,10);
    cout << "Selection Sort:   ";
    Print(buf);

    Copy(buf0, buf);
    InsertionSort(buf,10);
    cout << "Insertion Sort:   ";
    Print(buf);

    Copy(buf0, buf);
    InsertionSort2(buf,10);
    cout << "Insertion Sort 2: ";
    Print(buf);

    Copy(buf0, buf);
    BubbleSort(buf,10);
    cout << "Bubble Sort:      ";
    Print(buf);

    Copy(buf0, buf);
    MERGESORT<int> mergesort(100);
    mergesort.Sort(buf, 10);
    cout << "MERGESORT:        ";
    Print(buf);

    Copy(buf0, buf);
    QUICKSORT<int> quicksort;
    quicksort.Sort(buf, 10);
    cout << "QUICKSORT:        ";
    Print(buf);

    Copy(buf0, buf);
    HEAPSORT<int> heapsort;
    heapsort.Sort(buf, 10);
    cout << "HEAPSORT:         ";
    Print(buf);

    int target = 5;
    cout << "Binary Search " << target << ": "
         << BinarySearch(buf, 10, target) << endl;
}

//****************************************************************

