// TREEFILE.CPP

#include <fstream.h>
#include <string.h>

typedef char SOMETYPE[25];
#include "tree.h"

static void RecursiveSave (ofstream &outp, TREENODE *root);
static TREENODE *RecursiveLoad (ifstream &inp);

//****************************************************************

void SaveTree (char filename[], TREENODE *root)

// Opens a file for writing and saves the tree in it.

{
    ofstream outp(filename);
    if (!outp) {
        cerr << "Cannot create " << filename << endl;
        return;
    }
    RecursiveSave(outp, root);
}

//****************************************************************

TREENODE *LoadTree (char filename[])

// Opens a file for reading and loads the tree from it.
// If the file does not exist, returns null.

{
    ifstream inp(filename);
    if (!inp) return 0;
    return RecursiveLoad(inp);
}

//****************************************************************
//***         Recursive Save and Load functions                ***
//****************************************************************

static void RecursiveSave (ofstream &outp, TREENODE *root)

// Recursive utility function -- called from SaveTree;
// Assumes that outp is open for writing.

{
    outp << "[";
    if (root) {
        outp << endl;
        outp << root->data << endl;
        RecursiveSave (outp, root->left);
        RecursiveSave (outp, root->right);
    }
    outp << "]\n";  // An empty tree is saved as one line: []
}

//****************************************************************

static TREENODE *RecursiveLoad (ifstream &inp)

// Recursive utility function -- called from LoadTree

{
    char buf[80];

    while (inp.getline(buf, 80))  // Skip all lines before the  
        if (buf[0] == '[') break; //   first '['.

    if (buf[1] == ']')            // If the tree is empty...
        return 0;

    TREENODE *root = new TREENODE;
    inp.getline(buf, 80);         // Read the next line into buf
    strcpy(root->data, buf);      // Set node->data from buf
    root->left = RecursiveLoad(inp);
    root->right = RecursiveLoad(inp);
    inp.getline(buf, 80);         // Read the closing ']'
    if (buf[0] != ']')
        cerr << "Tree file format error: " << buf << endl;
    return root;
}

////////////////////////////////////////////////////////////////
// TEST.CPP

inline void Visit (TREENODE *node) { cout << node->data << ' '; }

void TraverseInOrder(TREENODE *root)

{
    if (!root) return;
    TraverseInOrder(root->left);
    Visit(root);
    TraverseInOrder(root->right);
}

//****************************************************************

void main()

{
    TREENODE *root = 0;

    Insert(root, "Mexico City");
    Insert(root, "Munich");
    Insert(root, "Montreal");
    Insert(root, "Moscow");
    Insert(root, "Los Angeles");
    Insert(root, "Seoul");
    Insert(root, "Barcelona");
    Insert(root, "Atlanta");
    Insert(root, "Sydney");

    TraverseInOrder(root);
    cout << "\n";
    SaveTree("tree.dat", root);
    Destroy(root);
    root = LoadTree("tree.dat");
    TraverseInOrder(root);
    cout << "\n";
    Destroy(root);
}

//****************************************************************
//*********                Compare, Copy      ********************
//****************************************************************

int Compare(const SOMETYPE data1, const SOMETYPE data2)

{
    return stricmp(data1, data2);
}

void Copy(const SOMETYPE data1, SOMETYPE data2)

{
    strcpy(data2, data1);
}


