typedef char SOMETYPE[25]; 

// TREE.H

#ifndef _TREE_H_
#define _TREE_H_

#include "bool.h"
#include "status.h"

struct TREENODE {
    SOMETYPE data;
    TREENODE *left;
    TREENODE *right;
};

typedef void CALLBACK(TREENODE *node);
    // Defines the CALLBACK data type as a function which takes one
    //   argument of the type TREENODE *...

class TREE {

  private:

    TREENODE *root;

    void RecursiveTraverseInOrder(TREENODE *node, CALLBACK Visit);
//    ...
    void Destroy(TREENODE *node);

  public:

    TREE() {root = 0;}
    STATUS Insert(const SOMETYPE data);
//    ...
    void TraverseInOrder(CALLBACK Visit) {
        RecursiveTraverseInOrder(root, Visit);
    }
//    ...
    ~TREE() {Destroy(root);}
};

int Compare(const SOMETYPE data1, const SOMETYPE data2);
void Copy(const SOMETYPE data1, SOMETYPE data2);

#endif // _TREE_H_

////////////////////////////////////////////////////////////////
// TREE.CPP

void TREE::Destroy(TREENODE *root)

{
    if (root) {
        Destroy(root->left);
        Destroy(root->right);
        delete root;
    }
}

STATUS TREE::Insert(const SOMETYPE data)

{
    TREENODE *&next = root;
    while (next) {
        int comp = Compare(data, next->data);
        if (comp == 0) // data is already in the tree
            return OK;
        else if (comp < 0)
            &next = &next->left;
        else // if (comp > 0)
            &next = &next->right;
    }

    // Allocate a new node and set next
    //   equal to the new node

    TREENODE *newnode = new TREENODE;
    if (!newnode) return FAILED;

    Copy (data, newnode->data);
    newnode->left =
    newnode->right = 0;
    next = newnode;
    return OK;
}

//...

void TREE::RecursiveTraverseInOrder(TREENODE *node, CALLBACK Visit)

{
    if (!node) return;
    RecursiveTraverseInOrder(node->left, Visit);
    Visit(node);
    RecursiveTraverseInOrder(node->right, Visit);
}

////////////////////////////////////////////////////////////////
// TEST.CPP

#include <iostream.h>
#include <string.h>

//typedef char SOMETYPE[25];
//#include "tree.h"

static void Visit (TREENODE *node) { cout << node->data << ' '; }

void main()

{
    TREE tree;

    tree.Insert("Mexico City");
    tree.Insert("Munich");
    tree.Insert("Montreal");
    tree.Insert("Moscow");
    tree.Insert("Los Angeles");
    tree.Insert("Seoul");
    tree.Insert("Barcelona");
    tree.Insert("Atlanta");
    tree.Insert("Sydney");

    cout << "Inorder:\n";
    tree.TraverseInOrder(Visit);
    cout << endl;
}

//****************************************************************
//*********                Compare, Copy      ********************
//****************************************************************

int Compare(const SOMETYPE data1, const SOMETYPE data2)

{
    return stricmp(data1, data2);
}

void Copy(const SOMETYPE data1, SOMETYPE data2)

{
    strcpy(data2, data1);
}

//****************************************************************
