// XTREE.H

#include "bool.h"

struct XNODE {
    char token[6];    // Represents either an operator or an operand.
    XNODE *left;
    XNODE *right;
};

inline bool IsLeaf(XNODE *node)

// Returns true if the node is a leaf.

{
    return (node && !node->left && !node->right);
}

inline bool IsOperator(XNODE *node)

{
    return (node && (node->left || node->right));
}

////////////////////////////////////////////////////////////////
// XTREE.CPP

#include <iostream.h>
//#include "xtree.h"

extern float GetValue(char operand[]);
extern float ApplyOperation(char op[], float x, float y);

float Eval(XNODE *root)

// Returns the value of the expression represented by
//   an expression tree.
// GetValue(...) and ApplyOperation(...) functions are
//   not defined here.

{
    float value;

    if (IsLeaf(root))              // root->token is an operand...
        value = GetValue(root->token);
    else                           // root->token is an operator...
        value = ApplyOperation(root->token,
                     Eval(root->left), Eval(root->right));
    return value;
} 

////////////////////////////////////////////////////////////////
// TEST.CPP

#include <iostream.h>
//#include "xtree.h"

/*

Expression tree for the expression

      (a+1)(bc + 5 - c)

            *
         /     \
        +       -
       / \     / \
      a   1   +   c
             / \
            *   5
           / \
          b   c


*/

static XNODE node1 =  {"b", 0, 0};
static XNODE node2 =  {"c", 0, 0};
static XNODE node3 =  {"*", &node1, &node2};
static XNODE node4 =  {"5", 0, 0};
static XNODE node5 =  {"a", 0, 0};
static XNODE node6 =  {"1", 0, 0};
static XNODE node7 =  {"+", &node3, &node4};
static XNODE node8 =  {"c", 0, 0};
static XNODE node9 =  {"+", &node5, &node6};
static XNODE node10 = {"-", &node7, &node8};
static XNODE node11 = {"*", &node9, &node10}; // root node

void PrintExprInOrder(XNODE *root)

// Prints out the expression in a fully parenthesized form
//   from an expression tree.

{
    if (root) {
        if (IsOperator(root->left)) cout << '(';
        PrintExprInOrder(root->left);
        if (IsOperator(root->left)) cout << ')';

        cout << ' ' << root->token << ' ';

        if (IsOperator(root->right)) cout << '(';
        PrintExprInOrder(root->right);
        if (IsOperator(root->right)) cout << ')';
    }
}

void PrintExprPreOrder(XNODE *root)

// Prints out an expression in prefix notation.

{
    if (root) {
        cout << root->token << ' ';
        PrintExprPreOrder(root->left);
        PrintExprPreOrder(root->right);
    }
}

void PrintExprPostOrder(XNODE *root)

// Prints out an expression in the postfix notation.

{
    if (root) {
        PrintExprPostOrder(root->left);
        PrintExprPostOrder(root->right);
        cout << root->token << ' ';
    }
}

void main()

{
    XNODE *root = &node11;

    cout << "Infix Notation:   ";
    PrintExprInOrder(root);
    cout << "\n\n";

    cout << "Prefix Notation:  ";
    PrintExprPreOrder(root);
    cout << "\n\n";

    cout << "Postfix Notation: ";
    PrintExprPostOrder(root);
    cout << "\n\n";
}

//****************************************************************

